# Test program to ask about destructors and PyQt4.

import sys, time
from PyQt4.QtGui import QApplication
from PyQt4.QtCore import QThread, QTimer, QObject, pyqtSignal

# A Worker that does nothing but wait as its computational task.
class Worker(QObject):
    def onStartWork(self):
        time.sleep(2)

class ThreadController(QObject):
    startWork = pyqtSignal()
    def __init__(self, parent):
        QObject.__init__(self, parent)
        # No parent object provided; if so, we get the error
        # "QObject::moveToThread: Cannot move objects with a parent".
        # Likewise, calling setParent after moving the object produces
        # "QObject::setParent: Cannot set parent, new parent is in a different
        # thread". So, we must manually invoke this object's destructor in
        # self.onParentDestroyed.
        self.worker = Worker()
        self.workerThread = QThread(self)
        self.worker.moveToThread(self.workerThread)

        # Since there's no way to invoke a method when the C++ destructor of
        # this class is called, approximate it by looking for the destroyed()
        # signal of the parent -- this object will soon have its destructor
        # invoked by the parent.
        parent.destroyed.connect(self.onParentDestroyed)
        self.startWork.connect(self.worker.onStartWork)
        self.workerThread.start()
        self.startWork.emit()

    # This is run shortly before this class's C++ destructor is invoked. It
    # emulates a C++ destructor by freeing resourced before the C++ class is
    # destroyed. Without this, we get nasty crashes since the Python portion
    # of the class is still alive. At a minimum, we see "QThread: Destroyed
    # while thread is still running" messages.
    def onParentDestroyed(self):
        print('onParentDestroyed')
        # Use deleteLater() to allow the worker to finish up any events in its
        # queue, then have its destructor invoked. Remove this class' reference
        # to it, allowing Python to garbage collect the Python half of the class
        # when the C++ part is deleted by deleteLater().
        self.worker.deleteLater()
        self.worker = None
        # Now, shut down the thread the Worker runs in.
        self.workerThread.quit()
        self.workerThread.wait()
        # Finally, detach (and probably garbage collect) the remaining object
        # used by this class.
        self.workerThread = None

if __name__ == '__main__':
    app = QApplication(sys.argv)

    # Exit the program shortly after the event loop starts up, but before the
    # thread finishes.
    QTimer.singleShot(500, app.exit)

    # Start a thread
    tm = ThreadController(app)

    # Run the main event loop.
    sys.exit(app.exec_())
